package sun.awt.windows;

import sun.java2d.DisposerRecord;
import java.awt.peer.DesktopPeer;
import java.awt.Desktop;
import sun.awt.AWTAccessor;
import java.awt.event.FocusEvent;
import java.awt.event.MouseEvent;
import java.awt.AWTEvent;
import javax.swing.text.JTextComponent;
import java.awt.TextComponent;
import java.awt.Toolkit;
import java.awt.RenderingHints;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.awt.AWTException;
import java.awt.dnd.MouseDragGestureRecognizer;
import java.awt.dnd.DragGestureRecognizer;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.InvalidDnDOperationException;
import java.awt.dnd.peer.DragSourceContextPeer;
import java.awt.dnd.DragGestureEvent;
import java.awt.EventQueue;
import java.util.concurrent.Executors;
import sun.awt.AppContext;
import java.awt.Dimension;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.Image;
import java.util.Locale;
import java.awt.font.TextAttribute;
import java.util.Map;
import java.awt.im.InputMethodHighlight;
import java.awt.im.spi.InputMethodDescriptor;
import sun.security.util.SecurityConstants;
import java.awt.datatransfer.Clipboard;
import sun.print.PrintJob2D;
import java.awt.PageAttributes;
import java.awt.JobAttributes;
import java.awt.PrintJob;
import java.util.Properties;
import sun.java2d.d3d.D3DRenderQueue;
import sun.java2d.opengl.OGLRenderQueue;
import sun.font.FontManager;
import sun.font.SunFontManager;
import sun.font.FontManagerFactory;
import java.awt.FontMetrics;
import java.awt.Font;
import sun.awt.Win32GraphicsEnvironment;
import sun.awt.Win32GraphicsDevice;
import java.awt.Insets;
import java.awt.HeadlessException;
import java.awt.peer.KeyboardFocusManagerPeer;
import sun.awt.datatransfer.DataTransferer;
import java.awt.peer.SystemTrayPeer;
import java.awt.SystemTray;
import java.awt.peer.TrayIconPeer;
import java.awt.TrayIcon;
import java.awt.peer.RobotPeer;
import java.awt.GraphicsDevice;
import java.awt.Robot;
import java.awt.peer.CheckboxMenuItemPeer;
import java.awt.CheckboxMenuItem;
import java.awt.peer.MenuItemPeer;
import java.awt.MenuItem;
import java.awt.peer.PopupMenuPeer;
import java.awt.PopupMenu;
import java.awt.peer.MenuPeer;
import java.awt.Menu;
import java.awt.peer.MenuBarPeer;
import java.awt.MenuBar;
import java.awt.peer.FileDialogPeer;
import java.awt.FileDialog;
import java.awt.peer.DialogPeer;
import java.awt.Dialog;
import java.awt.peer.WindowPeer;
import java.awt.Window;
import java.awt.peer.PanelPeer;
import java.awt.Panel;
import java.awt.peer.CanvasPeer;
import java.awt.Canvas;
import sun.awt.LightweightFrame;
import java.awt.peer.FramePeer;
import java.awt.Frame;
import java.awt.peer.ChoicePeer;
import java.awt.Choice;
import java.awt.peer.TextAreaPeer;
import java.awt.TextArea;
import java.awt.peer.ScrollPanePeer;
import java.awt.ScrollPane;
import java.awt.peer.ScrollbarPeer;
import java.awt.Scrollbar;
import java.awt.peer.CheckboxPeer;
import java.awt.Checkbox;
import java.awt.peer.ListPeer;
import java.awt.List;
import java.awt.peer.LabelPeer;
import java.awt.Label;
import java.awt.peer.TextFieldPeer;
import java.awt.TextField;
import java.awt.peer.ButtonPeer;
import java.awt.Button;
import sun.misc.ThreadGroupUtils;
import sun.awt.AWTAutoShutdown;
import sun.java2d.Disposer;
import sun.misc.PerformanceLogger;
import java.awt.GraphicsEnvironment;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.awt.Component;
import java.lang.ref.WeakReference;
import java.util.concurrent.ExecutorService;
import java.awt.image.ColorModel;
import java.awt.peer.FontPeer;
import java.util.Hashtable;
import java.awt.GraphicsConfiguration;
import sun.util.logging.PlatformLogger;
import sun.awt.SunToolkit;

public final class WToolkit extends SunToolkit implements Runnable
{
    private static final PlatformLogger log;
    public static final String XPSTYLE_THEME_ACTIVE = "win.xpstyle.themeActive";
    static GraphicsConfiguration config;
    WClipboard clipboard;
    private Hashtable<String, FontPeer> cacheFontPeer;
    private WDesktopProperties wprops;
    protected boolean dynamicLayoutSetting;
    private static boolean areExtraMouseButtonsEnabled;
    private static boolean loaded;
    private final Object anchor;
    private boolean inited;
    static ColorModel screenmodel;
    private static ExecutorService displayChangeExecutor;
    private static final String prefix = "DnD.Cursor.";
    private static final String postfix = ".32x32";
    private static final String awtPrefix = "awt.";
    private static final String dndPrefix = "DnD.";
    private static final WeakReference<Component> NULL_COMPONENT_WR;
    private volatile WeakReference<Component> compOnTouchDownEvent;
    private volatile WeakReference<Component> compOnMousePressedEvent;
    
    private static native void initIDs();
    
    public static void loadLibraries() {
        if (!WToolkit.loaded) {
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    System.loadLibrary("awt");
                    return null;
                }
            });
            WToolkit.loaded = true;
        }
    }
    
    private static native String getWindowsVersion();
    
    private static native void disableCustomPalette();
    
    public static void resetGC() {
        if (GraphicsEnvironment.isHeadless()) {
            WToolkit.config = null;
        }
        else {
            WToolkit.config = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
        }
    }
    
    public static native boolean embeddedInit();
    
    public static native boolean embeddedDispose();
    
    public native void embeddedEventLoopIdleProcessing();
    
    private static native void postDispose();
    
    private static native boolean startToolkitThread(Runnable p0, ThreadGroup p1);
    
    public WToolkit() {
        this.dynamicLayoutSetting = false;
        this.anchor = new Object();
        this.inited = false;
        this.compOnTouchDownEvent = WToolkit.NULL_COMPONENT_WR;
        this.compOnMousePressedEvent = WToolkit.NULL_COMPONENT_WR;
        if (PerformanceLogger.loggingEnabled()) {
            PerformanceLogger.setTime("WToolkit construction");
        }
        Disposer.addRecord(this.anchor, new WToolkit.ToolkitDisposer());
        AWTAutoShutdown.notifyToolkitThreadBusy();
        ThreadGroup group = (ThreadGroup)AccessController.doPrivileged(ThreadGroupUtils::getRootThreadGroup);
        if (!startToolkitThread(this, group)) {
            Thread thread = new Thread(group, this, "AWT-Windows");
            thread.setDaemon(true);
            thread.start();
        }
        try {
            synchronized (this) {
                while (!this.inited) {
                    this.wait();
                }
            }
        }
        catch (InterruptedException ex) {}
        this.setDynamicLayout(true);
        WToolkit.areExtraMouseButtonsEnabled = Boolean.parseBoolean(System.getProperty("sun.awt.enableExtraMouseButtons", "true"));
        System.setProperty("sun.awt.enableExtraMouseButtons", "" + WToolkit.areExtraMouseButtonsEnabled);
        setExtraMouseButtonsEnabledNative(WToolkit.areExtraMouseButtonsEnabled);
    }
    
    private final void registerShutdownHook() {
        AccessController.doPrivileged(()/*  */ -> {
            Thread hook = new Thread(ThreadGroupUtils.getRootThreadGroup(), this::shutdown);
            hook.setContextClassLoader(null);
            Runtime.getRuntime().addShutdownHook(hook);
            return null;
        });
    }
    
    @Override
    public void run() {
        AccessController.doPrivileged(()/*  */ -> {
            Thread.currentThread().setContextClassLoader(null);
            return null;
        });
        Thread.currentThread().setPriority(6);
        int init = this.init();
        if (!(init == 0)) {
            this.registerShutdownHook();
        }
        synchronized (this) {
            this.inited = true;
            this.notifyAll();
        }
        if (!(init == 0)) {
            this.eventLoop();
        }
    }
    
    private native boolean init();
    
    private native void eventLoop();
    
    private native void shutdown();
    
    static native void startSecondaryEventLoop();
    
    static native void quitSecondaryEventLoop();
    
    @Override
    public ButtonPeer createButton(Button target) {
        WButtonPeer peer = new WButtonPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public TextFieldPeer createTextField(TextField target) {
        WTextFieldPeer peer = new WTextFieldPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public LabelPeer createLabel(Label target) {
        WLabelPeer peer = new WLabelPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public ListPeer createList(List target) {
        WListPeer peer = new WListPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public CheckboxPeer createCheckbox(Checkbox target) {
        WCheckboxPeer peer = new WCheckboxPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public ScrollbarPeer createScrollbar(Scrollbar target) {
        WScrollbarPeer peer = new WScrollbarPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public ScrollPanePeer createScrollPane(ScrollPane target) {
        WScrollPanePeer peer = new WScrollPanePeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public TextAreaPeer createTextArea(TextArea target) {
        WTextAreaPeer peer = new WTextAreaPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public ChoicePeer createChoice(Choice target) {
        WChoicePeer peer = new WChoicePeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public FramePeer createFrame(Frame target) {
        WFramePeer peer = new WFramePeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public FramePeer createLightweightFrame(LightweightFrame target) {
        WLightweightFramePeer peer = new WLightweightFramePeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public CanvasPeer createCanvas(Canvas target) {
        WCanvasPeer peer = new WCanvasPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public void disableBackgroundErase(Canvas canvas) {
        WCanvasPeer wCanvasPeer = (WCanvasPeer)canvas.getPeer();
        if (wCanvasPeer == null) {
            throw new IllegalStateException("Canvas must have a valid peer");
        }
        wCanvasPeer.disableBackgroundErase();
        return;
    }
    
    @Override
    public PanelPeer createPanel(Panel target) {
        WPanelPeer peer = new WPanelPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public WindowPeer createWindow(Window target) {
        WWindowPeer peer = new WWindowPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public DialogPeer createDialog(Dialog target) {
        WDialogPeer peer = new WDialogPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public FileDialogPeer createFileDialog(FileDialog target) {
        WFileDialogPeer peer = new WFileDialogPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public MenuBarPeer createMenuBar(MenuBar target) {
        WMenuBarPeer peer = new WMenuBarPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public MenuPeer createMenu(Menu target) {
        WMenuPeer peer = new WMenuPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public PopupMenuPeer createPopupMenu(PopupMenu target) {
        WPopupMenuPeer peer = new WPopupMenuPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public MenuItemPeer createMenuItem(MenuItem target) {
        WMenuItemPeer peer = new WMenuItemPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public CheckboxMenuItemPeer createCheckboxMenuItem(CheckboxMenuItem target) {
        WCheckboxMenuItemPeer peer = new WCheckboxMenuItemPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    public RobotPeer createRobot(Robot robot, GraphicsDevice graphicsDevice) {
        return new WRobotPeer(graphicsDevice);
    }
    
    public WEmbeddedFramePeer createEmbeddedFrame(WEmbeddedFrame target) {
        WEmbeddedFramePeer peer = new WEmbeddedFramePeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    WPrintDialogPeer createWPrintDialog(WPrintDialog target) {
        WPrintDialogPeer peer = new WPrintDialogPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    WPageDialogPeer createWPageDialog(WPageDialog target) {
        WPageDialogPeer peer = new WPageDialogPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public TrayIconPeer createTrayIcon(TrayIcon target) {
        WTrayIconPeer peer = new WTrayIconPeer(target);
        SunToolkit.targetCreatedPeer(target, peer);
        return peer;
    }
    
    @Override
    public SystemTrayPeer createSystemTray(SystemTray systemTray) {
        return new WSystemTrayPeer(systemTray);
    }
    
    @Override
    public boolean isTraySupported() {
        return true;
    }
    
    @Override
    public DataTransferer getDataTransferer() {
        return WDataTransferer.getInstanceImpl();
    }
    
    @Override
    public KeyboardFocusManagerPeer getKeyboardFocusManagerPeer() throws HeadlessException {
        return WKeyboardFocusManagerPeer.getInstance();
    }
    
    private native void setDynamicLayoutNative(boolean p0);
    
    @Override
    public void setDynamicLayout(boolean dynamicLayoutSetting) {
        if (dynamicLayoutSetting == this.dynamicLayoutSetting) {
            return;
        }
        this.setDynamicLayoutNative(this.dynamicLayoutSetting = dynamicLayoutSetting);
        return;
    }
    
    @Override
    protected boolean isDynamicLayoutSet() {
        return this.dynamicLayoutSetting;
    }
    
    private native boolean isDynamicLayoutSupportedNative();
    
    @Override
    public boolean isDynamicLayoutActive() {
        return this.isDynamicLayoutSet() && this.isDynamicLayoutSupported();
    }
    
    @Override
    public boolean isFrameStateSupported(int n) {
        switch (n) {
            case 0:
            case 1:
            case 6: {
                return true;
            }
            default: {
                return false;
            }
        }
    }
    
    static native ColorModel makeColorModel();
    
    static ColorModel getStaticColorModel() {
        if (GraphicsEnvironment.isHeadless()) {
            throw new IllegalArgumentException();
        }
        if (WToolkit.config == null) {
            resetGC();
        }
        return WToolkit.config.getColorModel();
    }
    
    @Override
    public ColorModel getColorModel() {
        return getStaticColorModel();
    }
    
    @Override
    public Insets getScreenInsets(GraphicsConfiguration graphicsConfiguration) {
        return this.getScreenInsets(((Win32GraphicsDevice)graphicsConfiguration.getDevice()).getScreen());
    }
    
    @Override
    public int getScreenResolution() {
        return ((Win32GraphicsEnvironment)GraphicsEnvironment.getLocalGraphicsEnvironment()).getXResolution();
    }
    
    protected native int getScreenWidth();
    
    protected native int getScreenHeight();
    
    private native Insets getScreenInsets(int p0);
    
    @Override
    public FontMetrics getFontMetrics(Font font) {
        FontManager instance = FontManagerFactory.getInstance();
        if (instance instanceof SunFontManager && ((SunFontManager)instance).usePlatformFontMetrics()) {
            return WFontMetrics.getFontMetrics(font);
        }
        return super.getFontMetrics(font);
    }
    
    @Override
    public FontPeer getFontPeer(String s, int n) {
        String lowerCase = s.toLowerCase();
        if (null != this.cacheFontPeer) {
            FontPeer fontPeer = (FontPeer)this.cacheFontPeer.get(lowerCase + n);
            if (null != fontPeer) {
                return fontPeer;
            }
        }
        WFontPeer value = new WFontPeer(s, n);
        if (value != null) {
            if (null == this.cacheFontPeer) {
                this.cacheFontPeer = new Hashtable(5, 0.9f);
            }
            if (null != this.cacheFontPeer) {
                this.cacheFontPeer.put(lowerCase + n, value);
            }
        }
        return value;
    }
    
    private native void nativeSync();
    
    @Override
    public void sync() {
        this.nativeSync();
        OGLRenderQueue.sync();
        D3DRenderQueue.sync();
    }
    
    @Override
    public PrintJob getPrintJob(Frame frame, String s, Properties properties) {
        return this.getPrintJob(frame, s, null, null);
    }
    
    @Override
    public PrintJob getPrintJob(Frame frame, String doctitle, JobAttributes jobAttributes, PageAttributes pageAttributes) {
        if (frame == null) {
            throw new NullPointerException("frame must not be null");
        }
        PrintJob2D printJob2D = new PrintJob2D(frame, doctitle, jobAttributes, pageAttributes);
        if (!printJob2D.printDialog()) {
            printJob2D = null;
            return printJob2D;
        }
        return printJob2D;
    }
    
    @Override
    public native void beep();
    
    @Override
    public boolean getLockingKeyState(int n) {
        if (n != 20 && n != 144 && n != 145 && n != 262) {
            throw new IllegalArgumentException("invalid key for Toolkit.getLockingKeyState");
        }
        return this.getLockingKeyStateNative(n);
    }
    
    private native boolean getLockingKeyStateNative(int p0);
    
    @Override
    public void setLockingKeyState(int n, boolean b) {
        if (n != 20 && n != 144 && n != 145 && n != 262) {
            throw new IllegalArgumentException("invalid key for Toolkit.setLockingKeyState");
        }
        this.setLockingKeyStateNative(n, b);
        return;
    }
    
    private native void setLockingKeyStateNative(int p0, boolean p1);
    
    @Override
    public Clipboard getSystemClipboard() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SecurityConstants.AWT.ACCESS_CLIPBOARD_PERMISSION);
        }
        synchronized (this) {
            if (this.clipboard == null) {
                this.clipboard = new WClipboard();
            }
        }
        return this.clipboard;
    }
    
    @Override
    protected native void loadSystemColors(int[] p0);
    
    public static final Object targetToPeer(Object target) {
        return SunToolkit.targetToPeer(target);
    }
    
    public static final void targetDisposedPeer(Object target, Object peer) {
        SunToolkit.targetDisposedPeer(target, peer);
    }
    
    @Override
    public InputMethodDescriptor getInputMethodAdapterDescriptor() {
        return new WInputMethodDescriptor();
    }
    
    @Override
    public Map<TextAttribute, ?> mapInputMethodHighlight(InputMethodHighlight inputMethodHighlight) {
        return WInputMethod.mapInputMethodHighlight(inputMethodHighlight);
    }
    
    @Override
    public boolean enableInputMethodsForTextComponent() {
        return true;
    }
    
    @Override
    public Locale getDefaultKeyboardLocale() {
        Locale nativeLocale = WInputMethod.getNativeLocale();
        if (nativeLocale == null) {
            return super.getDefaultKeyboardLocale();
        }
        return nativeLocale;
    }
    
    @Override
    public Cursor createCustomCursor(Image image, Point point, String s) throws IndexOutOfBoundsException {
        return new WCustomCursor(image, point, s);
    }
    
    @Override
    public Dimension getBestCursorSize(int n, int n2) {
        return new Dimension(WCustomCursor.getCursorWidth(), WCustomCursor.getCursorHeight());
    }
    
    @Override
    public native int getMaximumCursorColors();
    
    static void paletteChanged() {
        ((Win32GraphicsEnvironment)GraphicsEnvironment.getLocalGraphicsEnvironment()).paletteChanged();
    }
    
    public static void displayChanged() {
        Runnable runnable = ()/*  */ -> ((Win32GraphicsEnvironment)GraphicsEnvironment.getLocalGraphicsEnvironment()).displayChanged();
        if (AppContext.getAppContext() != null) {
            EventQueue.invokeLater(runnable);
        }
        else {
            if (WToolkit.displayChangeExecutor == null) {
                WToolkit.displayChangeExecutor = Executors.newFixedThreadPool(1, runnable2/* java.lang.Runnable, */ -> {
                    Thread thread = Executors.defaultThreadFactory().newThread(runnable2);
                    thread.setDaemon(true);
                    return thread;
                });
            }
            WToolkit.displayChangeExecutor.submit(runnable);
        }
    }
    
    @Override
    public DragSourceContextPeer createDragSourceContextPeer(DragGestureEvent dragGestureEvent) throws InvalidDnDOperationException {
        LightweightFrame lightweightFrame = SunToolkit.getLightweightFrame(dragGestureEvent.getComponent());
        if (lightweightFrame != null) {
            return lightweightFrame.createDragSourceContextPeer(dragGestureEvent);
        }
        return WDragSourceContextPeer.createDragSourceContextPeer(dragGestureEvent);
    }
    
    @Override
    public <T extends DragGestureRecognizer> T createDragGestureRecognizer(Class<T> obj, DragSource dragSource, Component c, int n, DragGestureListener dragGestureListener) {
        LightweightFrame lightweightFrame = SunToolkit.getLightweightFrame(c);
        if (lightweightFrame != null) {
            return lightweightFrame.createDragGestureRecognizer(obj, dragSource, c, n, dragGestureListener);
        }
        if (MouseDragGestureRecognizer.class.equals(obj)) {
            return new WMouseDragGestureRecognizer(dragSource, c, n, dragGestureListener);
        }
        return null;
    }
    
    @Override
    protected Object lazilyLoadDesktopProperty(String name) {
        if (name.startsWith("DnD.Cursor.")) {
            String string = name.substring("DnD.Cursor.".length(), name.length()) + ".32x32";
            try {
                return Cursor.getSystemCustomCursor(string);
            }
            catch (AWTException cause) {
                throw new RuntimeException("cannot load system cursor: " + string, cause);
            }
        }
        else {
            if (name.equals("awt.dynamicLayoutSupported")) {
                return Boolean.valueOf(this.isDynamicLayoutSupported());
            }
            if (WDesktopProperties.isWindowsProperty(name) || name.startsWith("awt.") || name.startsWith("DnD.")) {
                synchronized (this) {
                    this.lazilyInitWProps();
                    return this.desktopProperties.get(name);
                }
            }
            return super.lazilyLoadDesktopProperty(name);
        }
    }
    
    private synchronized void lazilyInitWProps() {
        if (this.wprops == null) {
            this.wprops = new WDesktopProperties(this);
            this.updateProperties(this.wprops.getProperties());
        }
    }
    
    private synchronized boolean isDynamicLayoutSupported() {
        int dynamicLayoutSupportedNative = this.isDynamicLayoutSupportedNative();
        this.lazilyInitWProps();
        Boolean obj = (Boolean)this.desktopProperties.get("awt.dynamicLayoutSupported");
        if (WToolkit.log.isLoggable(PlatformLogger.Level.FINER)) {
            WToolkit.log.finer("In WTK.isDynamicLayoutSupported()   nativeDynamic == " + (boolean)dynamicLayoutSupportedNative + "   wprops.dynamic == " + obj);
        }
        if (obj == null || dynamicLayoutSupportedNative != (((boolean)obj) ? 1 : 0)) {
            this.windowsSettingChange();
            return dynamicLayoutSupportedNative;
        }
        return (boolean)obj;
    }
    
    private void windowsSettingChange() {
        Map<String, Object> wProps = this.getWProps();
        if (wProps == null) {
            return;
        }
        this.updateXPStyleEnabled(wProps.get("win.xpstyle.themeActive"));
        if (AppContext.getAppContext() == null) {
            this.updateProperties(wProps);
        }
        else {
            EventQueue.invokeLater(()/*  */ -> this.updateProperties(var_1_04));
        }
        return;
    }
    
    private synchronized void updateProperties(Map<String, Object> map) {
        if (null == map) {
            return;
        }
        this.updateXPStyleEnabled(map.get("win.xpstyle.themeActive"));
        for (String s : map.keySet()) {
            Object value = map.get(s);
            if (WToolkit.log.isLoggable(PlatformLogger.Level.FINER)) {
                WToolkit.log.finer("changed " + s + " to " + value);
            }
            this.setDesktopProperty(s, value);
        }
        return;
    }
    
    private synchronized Map<String, Object> getWProps() {
        return (this.wprops != null) ? this.wprops.getProperties() : null;
    }
    
    private void updateXPStyleEnabled(Object obj) {
        ThemeReader.xpStyleEnabled = Boolean.TRUE.equals(obj);
    }
    
    @Override
    public synchronized void addPropertyChangeListener(String name, PropertyChangeListener pcl) {
        if (name == null) {
            return;
        }
        if (WDesktopProperties.isWindowsProperty(name) || name.startsWith("awt.") || name.startsWith("DnD.")) {
            this.lazilyInitWProps();
        }
        super.addPropertyChangeListener(name, pcl);
        return;
    }
    
    @Override
    protected synchronized void initializeDesktopProperties() {
        this.desktopProperties.put("DnD.Autoscroll.initialDelay", Integer.valueOf(50));
        this.desktopProperties.put("DnD.Autoscroll.interval", Integer.valueOf(50));
        this.desktopProperties.put("DnD.isDragImageSupported", Boolean.TRUE);
        this.desktopProperties.put("Shell.shellFolderManager", "sun.awt.shell.Win32ShellFolderManager2");
    }
    
    @Override
    protected synchronized RenderingHints getDesktopAAHints() {
        if (this.wprops == null) {
            return null;
        }
        return this.wprops.getDesktopAAHints();
    }
    
    @Override
    public boolean isModalityTypeSupported(Dialog.ModalityType modalityType) {
        return modalityType == null || modalityType == Dialog.ModalityType.MODELESS || modalityType == Dialog.ModalityType.DOCUMENT_MODAL || modalityType == Dialog.ModalityType.APPLICATION_MODAL || modalityType == Dialog.ModalityType.TOOLKIT_MODAL;
    }
    
    @Override
    public boolean isModalExclusionTypeSupported(Dialog.ModalExclusionType modalExclusionType) {
        return modalExclusionType == null || modalExclusionType == Dialog.ModalExclusionType.NO_EXCLUDE || modalExclusionType == Dialog.ModalExclusionType.APPLICATION_EXCLUDE || modalExclusionType == Dialog.ModalExclusionType.TOOLKIT_EXCLUDE;
    }
    
    public static WToolkit getWToolkit() {
        return (WToolkit)Toolkit.getDefaultToolkit();
    }
    
    @Override
    public boolean useBufferPerWindow() {
        return !Win32GraphicsEnvironment.isDWMCompositionEnabled();
    }
    
    @Override
    public void grab(Window window) {
        if (window.getPeer() != null) {
            ((WWindowPeer)window.getPeer()).grab();
        }
    }
    
    @Override
    public void ungrab(Window window) {
        if (window.getPeer() != null) {
            ((WWindowPeer)window.getPeer()).ungrab();
        }
    }
    
    private boolean isComponentValidForTouchKeyboard(Component component) {
        return component != null && component.isEnabled() && component.isFocusable() && ((component instanceof TextComponent && ((TextComponent)component).isEditable()) || (component instanceof JTextComponent && ((JTextComponent)component).isEditable()));
    }
    
    @Override
    public void showOrHideTouchKeyboard(Component component, AWTEvent awtEvent) {
        if (!(component instanceof TextComponent) && !(component instanceof JTextComponent)) {
            return;
        }
        if (awtEvent instanceof MouseEvent && this.isComponentValidForTouchKeyboard(component)) {
            MouseEvent mouseEvent = (MouseEvent)awtEvent;
            if (mouseEvent.getID() == 501) {
                if (AWTAccessor.getMouseEventAccessor().isCausedByTouchEvent(mouseEvent)) {
                    this.compOnTouchDownEvent = new WeakReference(component);
                }
                else {
                    this.compOnMousePressedEvent = new WeakReference(component);
                }
            }
            else if (mouseEvent.getID() == 502) {
                if (AWTAccessor.getMouseEventAccessor().isCausedByTouchEvent(mouseEvent)) {
                    if (this.compOnTouchDownEvent.get() == component) {
                        this.showTouchKeyboard(true);
                    }
                    this.compOnTouchDownEvent = WToolkit.NULL_COMPONENT_WR;
                }
                else {
                    if (this.compOnMousePressedEvent.get() == component) {
                        this.showTouchKeyboard(false);
                    }
                    this.compOnMousePressedEvent = WToolkit.NULL_COMPONENT_WR;
                }
            }
        }
        else if ((awtEvent instanceof FocusEvent)) {
            FocusEvent focusEvent = (FocusEvent)awtEvent;
            if (focusEvent.getID() == 1005 && !this.isComponentValidForTouchKeyboard(focusEvent.getOppositeComponent())) {
                this.hideTouchKeyboard();
            }
        }
        return;
    }
    
    private native void showTouchKeyboard(boolean p0);
    
    private native void hideTouchKeyboard();
    
    public native boolean syncNativeQueue(long p0);
    
    @Override
    public boolean isDesktopSupported() {
        return true;
    }
    
    @Override
    public DesktopPeer createDesktopPeer(Desktop desktop) {
        return new WDesktopPeer();
    }
    
    private static native void setExtraMouseButtonsEnabledNative(boolean p0);
    
    @Override
    public boolean areExtraMouseButtonsEnabled() throws HeadlessException {
        return WToolkit.areExtraMouseButtonsEnabled;
    }
    
    private synchronized native int getNumberOfButtonsImpl();
    
    @Override
    public int getNumberOfButtons() {
        if (WToolkit.numberOfButtons == 0) {
            WToolkit.numberOfButtons = this.getNumberOfButtonsImpl();
        }
        return (WToolkit.numberOfButtons > 20) ? 20 : WToolkit.numberOfButtons;
    }
    
    @Override
    public boolean isWindowOpacitySupported() {
        return true;
    }
    
    @Override
    public boolean isWindowShapingSupported() {
        return true;
    }
    
    @Override
    public boolean isWindowTranslucencySupported() {
        return true;
    }
    
    @Override
    public boolean isTranslucencyCapable(GraphicsConfiguration graphicsConfiguration) {
        return true;
    }
    
    @Override
    public boolean needUpdateWindow() {
        return true;
    }
    
    static void access$000() {
        disableCustomPalette();
    }
    
    static void access$100() {
        postDispose();
    }
    
    static {
        log = PlatformLogger.getLogger("sun.awt.windows.WToolkit");
        WToolkit.areExtraMouseButtonsEnabled = true;
        WToolkit.loaded = false;
        loadLibraries();
        initIDs();
        if (WToolkit.log.isLoggable(PlatformLogger.Level.FINE)) {
            WToolkit.log.fine("Win version: " + getWindowsVersion());
        }
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                String property = System.getProperty("browser");
                if (property != null && property.equals("sun.plugin")) {
                    WToolkit.access$000();
                }
                return null;
            }
        });
        NULL_COMPONENT_WR = new WeakReference(null);
    }
    
    static class ToolkitDisposer implements DisposerRecord
    {
        @Override
        public void dispose() {
            WToolkit.access$100();
        }
    }
}
